/********************************************************************************************************
 * PROGRAM      : fcp
 * DATE - TIME  : dimanche 10 juin 2007 - 19h14
 * AUTHOR       : Michal Todorovic ( michael.todorovic free.fr )
 * FILENAME     : hal.cpp
 * LICENSE      : GPLv2
 * COMMENTARY   : Source code for using the C hal library
 ********************************************************************************************************/
#include "hal.h"

hal::hal()
{
msg = new msgConsole();
}

hal::~hal()
{
delete msg;
}

bool hal::initHal()
{
	m_halContext = libhal_ctx_new();
	if(!m_halContext)
		msg->critical(tr("Failed to initialize HAL !"));
	else
		msg->debug(tr("HAL context created"));

	dbus_error_init(&error);
	dbus_connection = dbus_bus_get(DBUS_BUS_SYSTEM, &error);
	if (dbus_error_is_set(&error))
	{
		dbus_error_free(&error);
		libhal_ctx_free(m_halContext);
		m_halContext = 0;
		msg->critical(tr("Failed to connect to DBUS !"));
		return false;
	}
	else
		msg->debug(tr("DBUS connection opened"));

	if (m_halContext)
	{
		libhal_ctx_set_dbus_connection(m_halContext, dbus_connection);
		dbus_error_init(&error);
		if (!libhal_ctx_init(m_halContext, &error))
		{
			printf("error %s %s\n", error.name, error.message);
			if (dbus_error_is_set(&error))
				dbus_error_free(&error);
			libhal_ctx_free(m_halContext);
			m_halContext = 0;
			return false;
		}
	}

return true;
}

char* hal::getStringProperty(char* where, char* object)
{
return libhal_device_get_property_string(m_halContext, where, object, &error);
}

