/********************************************************************************************************
 * PROGRAM      : fcp
 * DATE - TIME  : dimanche 10 juin 2007 - 15h40
 * AUTHOR       : Michal Todorovic ( michael.todorovic@free.fr )
 * FILENAME     : mainUi.cpp
 * LICENSE      : GPLv2
 * COMMENTARY   : Source code for the main window
 ********************************************************************************************************/
#include "mainUi.h"

#include <QCloseEvent>
#include <QFile>

QPointer<mainUi> mainUi::_self = 0L;

mainUi* mainUi::self( QWidget* parent )
{
	if ( !_self )
		_self = new mainUi( parent );
	return _self;
}

mainUi::mainUi( QWidget* parent )
	: QWidget( parent )
{
	setupUi( this );
	msg = new msgConsole();
	msg->verbose = 1;

	halObj = new hal();
	getHalInfo();
	getBootConfig();

	displayObj = new displayClass();

	DcomboBox->setCurrentIndex(displayObj->getDefaultDepth(0));
	resolutions = displayObj->getAvailableResolutions();

	connect(Bpush_cancel, SIGNAL(clicked()), this, SLOT(cancelBootChanges()));
	connect(Bpush_apply, SIGNAL(clicked()), this, SLOT(applyBootChanges()));
	connect(Blist_boot, SIGNAL(itemDoubleClicked(QListWidgetItem*)), this, SLOT(editItem(QListWidgetItem*)));
}

void mainUi::closeEvent( QCloseEvent* e )
{
	e->accept();
}

void mainUi::getHalInfo(void)
{
if(!halObj->initHal())
	{
		msg->critical(tr("HAL init failed !"));
	}
	else
	{
		halObj->formFactor = tr(halObj->getStringProperty("/org/freedesktop/Hal/devices/computer", "system.formfactor"));
		Slbl_systype_value->setText(halObj->formFactor);

		halObj->kernel_version = halObj->getStringProperty("/org/freedesktop/Hal/devices/computer", "system.kernel.version");
		halObj->kernel_type = halObj->getStringProperty("/org/freedesktop/Hal/devices/computer", "system.kernel.machine");
		Slbl_kernel_value->setText(halObj->kernel_version + " " + tr("on") + " " + halObj->kernel_type);

		QString out = halObj->getStringProperty("/org/freedesktop/Hal/devices/acpi_CPU0", "info.product");
		if(out == NULL)
			Slbl_processor_value->setText(tr("Not found !"));
		else
			Slbl_processor_value->setText(out);
	}
}

void mainUi::getBootConfig(void)
{
	int defaultValue=-1;
	int lines = 0;
	int i;
	QRegExp defaultVal;
	QRegExp title;
	QRegExp time;

	QFile grubConfig("/boot/grub/menu.lst");
	if (!grubConfig.open(QIODevice::ReadOnly | QIODevice::Text))
		msg->critical(tr("Failed to open grub config in read only mode !"));

	defaultVal.setPattern("^default.*([0-9]*)\n");
	title.setPattern("^title[\\t-\\s]*([^\t]+)\n");
	time.setPattern("^timeout.*([0-9]*)\n");

	msg->debug(tr("Operating systems found in configuration"));
	while (!grubConfig.atEnd())
	{
		lines++;
		QByteArray line = grubConfig.readLine();

		defaultVal.indexIn(line);
		title.indexIn(line);
		time.indexIn(line);

		if(defaultVal.capturedTexts().at(1) != NULL)
			{
			defaultValue = defaultVal.capturedTexts().at(1).toInt();
			bootCfg.str_defaultValue = defaultVal.capturedTexts().at(0);
			bootCfg.defaultValue = defaultValue;
			}		

		if(title.capturedTexts().at(1) != NULL)
			{
			msg->debug(QString("\t- ") + title.capturedTexts().at(1).simplified());
			bootCfg.bootConfig << title.capturedTexts().at(1).simplified();
			bootCfg.bootConfigOld << title.capturedTexts().at(0);
			}

		if(time.capturedTexts().at(1) != NULL)
			{
			bootCfg.timeout = time.capturedTexts().at(1).toInt();
			bootCfg.str_timeout = time.capturedTexts().at(0);
			}
	}
	grubConfig.close();
	if(defaultValue == -1)
	{
		msg->warning(tr("Booting default value not found ! Defaulting to 0"));
		defaultValue = 0;
	}

	bootCfg.defaultValue = defaultValue;
	msg->debug(tr("Default booting value is %n", "", defaultValue));
	msg->debug(tr("Parsing /boot/grub/menu.lst done. %n lines parsed", "", lines));

	for(i=0;i<bootCfg.bootConfig.size();i++)
		Blist_boot->addItem(bootCfg.bootConfig.at(i));

	Blist_boot->setCurrentRow(defaultValue);
	Bspin_timeout->setValue(bootCfg.timeout);

	connect(Blist_boot, SIGNAL(currentRowChanged(int)), this, SLOT(bootConfigChanged()));
	connect(Bspin_timeout, SIGNAL(valueChanged(int)), this, SLOT(bootConfigChanged()));
}

void mainUi::bootConfigChanged()
{
	Bpush_apply->setEnabled(true);
	Bpush_cancel->setEnabled(true);
}

void mainUi::cancelBootChanges()
{
	Blist_boot->setCurrentRow(bootCfg.defaultValue);
	Bspin_timeout->setValue(bootCfg.timeout);
	Bpush_apply->setEnabled(false);
	Bpush_cancel->setEnabled(false);
}

void mainUi::applyBootChanges()
{
	QFile grubConfig("/boot/grub/menu.lst");
	QString replaceStr;
	QByteArray line;
	int bytes;

	if (!grubConfig.open(QIODevice::ReadWrite | QIODevice::Text))
		msg->gCritical(tr("Failed to open grub config in read write mode !"));

	line = grubConfig.readAll();
	grubConfig.close();
	if (!grubConfig.open(QIODevice::ReadWrite | QIODevice::Text | QIODevice::Truncate))
		msg->gCritical(tr("Failed to open grub config in truncate mode !"));

	line.replace(bootCfg.str_defaultValue, qPrintable(QString("default\t\t") + QString::number(Blist_boot->currentRow()) + QString("\n")));
	line.replace(bootCfg.str_timeout, qPrintable(QString("timeout\t\t") + QString::number(Bspin_timeout->value()) + QString("\n")));
	if(bootCfg.oldName != NULL)
		for(int i=0;i<bootCfg.bootConfig.size();i++)
			line.replace(bootCfg.bootConfigOld.at(i), qPrintable(QString("title\t\t") + bootCfg.bootConfig.at(i) + QString("\n")));

	grubConfig.seek(0);
	bytes = grubConfig.write(line);
	grubConfig.close();
	if(!bytes || bytes == -1)
		{
		msg->gCritical(tr("Grub configuration write error !"));
		}
	else
		{
		msg->debug(tr("Configuration saved !"));
		Bpush_apply->setEnabled(false);
		Bpush_cancel->setEnabled(false);		
		}
}

void mainUi::editItem(QListWidgetItem* item)
{
	bootCfg.editedRow = Blist_boot->currentRow();
	rename = new changeNameDialog();
	rename->show();
	bootCfg.oldName = item->text();
	rename->lineEdit->setText(bootCfg.oldName);
	connect(rename->buttonBox, SIGNAL(accepted()), this, SLOT(getNewName()));
}

void mainUi::getNewName()
{
	Blist_boot->item(bootCfg.editedRow)->setText(rename->lineEdit->text());
	bootCfg.bootConfig.replaceInStrings(bootCfg.oldName, rename->lineEdit->text(), Qt::CaseSensitive);
	Bpush_apply->setEnabled(true);
	Bpush_cancel->setEnabled(true);			
}
